/* ==========================================================================*/
/* - - - - - - - - - -  T I M E R S , C  - - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   07-10-2004   M. Wheatley                                       */
/*	Modified  07-18-2004   MW                                                */
/*.........................................................................  */
/* TIMERS Module                                                             */
/*.........................................................................  */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include <p30f6014.h>
#include <timer.h>
#include <dsp.h>
#include "common.h"
#include "timers.h"

/*---------------------------------------------------------------------------*/
/*----------------> P R O C E S S O R    S P E C I F I C <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define NVMCON_WRITE_1_EEPROM   0x4004
#define NVMCON_ERASE_1_EEPROM   0x4044
#define EE_DATA_PAGE 0x7F		//start of EEDATA section

/*---------------------------------------------------------------------------*/
/*----------------> E X T E R N A L  R E F E R E N C E S <-------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*--------------> G L O B A L  M E M O R Y  A L L O C A T I O N <------------*/
/*---------------------------------------------------------------------------*/
volatile struct SEM_STRUCA SemA; //global arrays of generel purpose semaphores
volatile struct SEM_STRUCB SemB;
volatile unsigned char KeyNum;
volatile unsigned char LCDHoldoffCounter;

/*---------------------------------------------------------------------------*/
/*--------------> L O C A L  M E M O R Y  A L L O C A T I O N <--------------*/
/*---------------------------------------------------------------------------*/
static volatile unsigned int DelayCounter;
static volatile unsigned int LastSwitch;

/*---------------------------------------------------------------------------*/
/*-----------------------> P R O T O T Y P E S <-----------------------------*/
/*---------------------------------------------------------------------------*/
void InitTimers( void );
int ReadEEData( unsigned int adr );
void WriteEEData( unsigned int adr , int data);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                          ---------------------                            */
/*                         | I n i t T i m e r s |                           */
/*                          ---------------------                            */
/*    This initializes the Timers used by various sub systems.               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitTimers( void )
{
	TRISAbits.TRISA15 = 1;	//Key Switch Input port bits
	TRISAbits.TRISA14 = 1;
	TRISAbits.TRISA13 = 1;
	TRISAbits.TRISA12 = 1;
	/* Enable Timer1 Interrupt and Priority to "1" */
	ConfigIntTimer1(T1_INT_PRIOR_1 & T1_INT_ON);
	WriteTimer1(0);
	OpenTimer1(T1_ON & T1_GATE_OFF & T1_IDLE_STOP &
				T1_PS_1_256 & T1_SYNC_EXT_OFF &
				T1_SOURCE_INT, CLOCK_FREQ/(256*10) );	//10Hz rate
	SemA.Timer1 = 0;
	DelayCounter = 0;
	LCDHoldoffCounter = 0;
	/* Enable Timer2 Interrupt and Priority to "2" */
	ConfigIntTimer2(T2_INT_PRIOR_2 & T2_INT_ON);
	WriteTimer2(0);
	OpenTimer2(T2_ON & T2_GATE_OFF & T2_IDLE_STOP &
				T2_PS_1_256 & T2_32BIT_MODE_OFF &
				T2_SOURCE_INT, CLOCK_FREQ/(256*25) );	//25Hz rate
	SemA.KeySwitch = 0;
	LastSwitch = PORTA & 0xF000;
	KeyNum = 0;
}

/*---------------------------------------------------------------------------*/
/*----------> I N T E R R U P T    S E R V I C E    R O U T I N E S <--------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | T 1 I n t e r r u p t |                              */
/*                     -----------------------                               */
/*  10Hz Timer1 Interrupt Service routine.                                   */
/* max .5uSec execution @ 4xPLL                                               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void __attribute__((__interrupt__)) _T1Interrupt(void)
{
	SemA.Timer1 = 1;
	IFS0bits.T1IF = 0; /* Clear Timer interrupt flag */
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | T 2 I n t e r r u p t |                              */
/*                     -----------------------                               */
/*  Timer2 Interrupt Service routine samples keys at 25Hz rate.              */
/* max 5uSec execution @ 4xPLL                                               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void __attribute__((__interrupt__)) _T2Interrupt(void)
{
	if(LastSwitch != (PORTA & 0xF000) )	//if any key changed state
	{
		LastSwitch = PORTA & 0xF000;
		if( LastSwitch != 0xF000)	//if any key has been depressed
		{
			if( !PORTAbits.RA15 )
				KeyNum = KEY4;
			else if( !PORTAbits.RA14 )
				KeyNum = KEY3;
			else if( !PORTAbits.RA13 )
				KeyNum = KEY2;
			else
				KeyNum = KEY1;
			SemA.KeySwitch = TRUE;
		}
	}
	if(DelayCounter>0)	//general purpose delay counter 40mS per tic
		DelayCounter--;
	if(LCDHoldoffCounter>0)
		LCDHoldoffCounter--;
	IFS0bits.T2IF = 0; /* Clear Timer2 interrupt flag */
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------                                   */
/*                    | T i m e D e l a y |                                  */
/*                     -------------------                                   */
/*  General Purpose Timer that waits N tics of Timer2.                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void TimeDelay(unsigned int Tics40ms)
{
	DelayCounter = Tics40ms;
	while(DelayCounter){};
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | R e a d E E D a t a   |                              */
/*                    | W r i t e E E D a t a |                              */
/*                     -----------------------                               */
/*  EEROM read write utilities.                                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int ReadEEData( unsigned int adr )
{					   
	TBLPAG = EE_DATA_PAGE;
	WREG0 = adr;
	asm volatile ("tblrdl [w0],w1");
	return WREG1;
}

void WriteEEData( unsigned int adr , int data)
{
	IFS0bits.NVMIF = 0;
	TBLPAG = EE_DATA_PAGE;
	NVMADR = adr;
	NVMCON = NVMCON_ERASE_1_EEPROM;
    asm volatile ( "push	SR" );
    asm volatile ( "mov	#0x00E0, w0" );
    asm volatile ( "ior	SR" );
    asm volatile( "mov #0x55, w0" );
    asm volatile( "mov w0, NVMKEY" );
    asm volatile( "mov #0xAA, w0" );
    asm volatile( "mov w0, NVMKEY" );
    NVMCONbits.WR = 1;
    asm volatile ( "pop	SR" );
    while (NVMCONbits.WR == 1){};
	//
	IFS0bits.NVMIF = 0;
	TBLPAG = EE_DATA_PAGE;
	WREG3 = data;
	WREG4 = adr;
	asm volatile("tblwtl w3,[w4]");
	NVMCON = NVMCON_WRITE_1_EEPROM;
    asm volatile ( "push	SR" );
    asm volatile ( "mov	#0x00E0, w0" );
    asm volatile ( "ior	SR" );
    asm volatile( "mov #0x55, w0" );
    asm volatile( "mov w0, NVMKEY" );
    asm volatile( "mov #0xAA, w0" );
    asm volatile( "mov w0, NVMKEY" );
    NVMCONbits.WR = 1;
    asm volatile ( "pop	SR" );
    while (NVMCONbits.WR == 1){};
    NVMCONbits.WREN = 0; 
}
